`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/04/22 19:35:26
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

module topmodule(
    clk,
    rst,
    vgaRed,
    vgaGreen,
    vgaBlue,
    hsync,
    vsync,
    PS2_DATA,
    PS2_CLK,
    rst_for_calc
);

    input clk; // clock from the crystal
    input rst;
    input rst_for_calc; 
    output reg [3:0] vgaRed, vgaGreen, vgaBlue;
    output hsync, vsync;

    inout PS2_CLK, PS2_DATA;

    wire clk_25M, clk_22;

    wire [511:0] key_down;
    wire [8:0] last_change;
    wire key_valid;
    wire [3:0] decode_result;

    wire valid_signal;

    wire [1:0] instruction;

    wire [3:0] digit_0, digit_1, digit_2, digit_3;
    wire [3:0] add_digit_2, add_digit_1, add_digit_0;
    wire [3:0] subtract_digit_1, subtract_digit_0;
    wire [3:0] multiply_digit_3, multiply_digit_2, multiply_digit_1, multiply_digit_0;

    wire [6:0] value_1, value_0;

    wire [2:0] curr_state;

    wire hsync, vsync, valid;
    wire [9:0] h_cnt, v_cnt; // 640 x 480
    wire [16:0] pixel_addr;

    wire [11:0] data;
    wire [11:0] pixel_0;
    wire [11:0] pixel_1;
    wire [11:0] pixel_2;
    wire [11:0] pixel_3;
    wire [11:0] pixel_4;
    wire [11:0] pixel_5;
    wire [11:0] pixel_6;
    wire [11:0] pixel_7;
    wire [11:0] pixel_8;
    wire [11:0] pixel_9;
    wire [11:0] pixel_add;
    wire [11:0] pixel_subtract;
    wire [11:0] pixel_multiply;
    wire [11:0] pixel_eq;

    assign valid_signal = key_down[last_change];

    clock_divisor U0(
    .clk1(clk_25M), 
    .clk(clk), 
    .clk22(clk_22)
    );

    KeyboardDecoder U1(
	.key_down(key_down),
	.last_change(last_change),
	.key_valid(key_valid),
	.PS2_DATA(PS2_DATA),
	.PS2_CLK(PS2_CLK),
	.rst(rst),
	.clk(clk)
    );

    key_decoder U2(
    .result(decode_result), 
    .select_in(last_change),
    .clk(clk),
    .rst(rst)
    );

    fsm U3(
    .instruction(instruction),
    .valid(valid_signal),
    .curr_state(curr_state),
    .rst(rst_for_calc),
    .decoded_input(decode_result)
);
    value_temp_reg U4(
    .digit_0(digit_0),
    .digit_1(digit_1),
    .digit_2(digit_2),
    .digit_3(digit_3),
    .state_sel(curr_state),
    .valid(valid_signal),
    .decoded_input(decode_result),
    .rst(rst_for_calc)
);
    dec_adder U5(
    .cout(add_digit_2), 
    .num_0(digit_0), 
    .num_1(digit_1),
    .num_2(digit_2),
    .num_3(digit_3),
    .digit_0_out(add_digit_0),
    .digit_1_out(add_digit_1)
    );

    dec_subtractor U6(
    .num_0(digit_0), 
    .num_1(digit_1),
    .num_2(digit_2),
    .num_3(digit_3),
    .digit_0_out(subtract_digit_0),
    .digit_1_out(subtract_digit_1),
    .value_0(value_0),
    .value_1(value_1)
    );

    dec_multiplier U7(
    .num_0(digit_0), 
    .num_1(digit_1),
    .num_2(digit_2),
    .num_3(digit_3),
    .digit_0_out(multiply_digit_0),
    .digit_1_out(multiply_digit_1),
    .digit_2_out(multiply_digit_2),
    .digit_3_out(multiply_digit_3)
    );


    vga_controller U8(
    .pclk(clk_25M),
    .reset(rst),
    .hsync(hsync),
    .vsync(vsync),
    .valid(valid),
    .h_cnt(h_cnt),
    .v_cnt(v_cnt)
    );

    mem_addr_gen U9(
    .h_cnt(h_cnt),
    .v_cnt(v_cnt),
    .pixel_addr(pixel_addr)
);
    blk_mem_gen_0 M0(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_0)
);
    blk_mem_gen_1 M1(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_1)
);
    blk_mem_gen_2 M2(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_2)
);
    blk_mem_gen_3 M3(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_3)
);
    blk_mem_gen_4 M4(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_4)
);
    blk_mem_gen_5 M5(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_5)
);
    blk_mem_gen_6 M6(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_6)
);
    blk_mem_gen_7 M7(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_7)
);
    blk_mem_gen_8 M8(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_8)
);
    blk_mem_gen_9 M9(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_9)
);
    blk_mem_gen_add M_add(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_add)
);
    blk_mem_gen_subtract M_subtract(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_subtract)
);
    blk_mem_gen_multiply M_multiply(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_multiply)
);
    blk_mem_gen_eq M_eq(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel_eq)
);
    always @* begin
        if (v_cnt < 128 || v_cnt > 256)
            {vgaRed, vgaGreen, vgaBlue} = 12'h0;
        else if (h_cnt < 64) //digit
            case (digit_3)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
            endcase
        else if (h_cnt < 128) //digit
            case (digit_2)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
            endcase
        else if (h_cnt < 192) //operator
            case (instruction)
                2'b00:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_add : 12'h0;
                2'b01:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_subtract : 12'h0;
                2'b10:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_multiply : 12'h0;
                default:
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
            endcase
        else if (h_cnt < 256) //digit
            case (digit_1)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
            endcase

        else if (h_cnt < 320) //digit
            case (digit_0)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
            endcase

        else if (h_cnt < 384) //equal sign
            {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_eq : 12'h0;

        else if (h_cnt < 448) //if doing multiply, then it will display. Otherwise, no display
            if (instruction == 2'b10)
                case (multiply_digit_3)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
            endcase
            else
                {vgaRed, vgaGreen, vgaBlue} = 12'h0;

        else if (h_cnt < 512) //if doing subtract, then it will not display
            if (instruction == 2'b00)
                case (add_digit_2)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
                endcase
            else if (instruction == 2'b01)
                if (value_1 >= value_0) 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
                else
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_subtract : 12'h0;
            else if (instruction == 2'b10)
                case (multiply_digit_2)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
                endcase
            else
                {vgaRed, vgaGreen, vgaBlue} = 12'h0;

        else if (h_cnt < 576)
            if (instruction == 2'b00)
                case (add_digit_1)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
                endcase
            else if (instruction == 2'b01)
                case (subtract_digit_1)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
                endcase
            else if (instruction == 2'b10)
                case (multiply_digit_1)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
                endcase
            else
                {vgaRed, vgaGreen, vgaBlue} = 12'h0;

        else if (h_cnt < 640)
            if (instruction == 2'b00)
                case (add_digit_0)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
                endcase
            else if (instruction == 2'b01)
                case (subtract_digit_0)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
                endcase
            else if (instruction == 2'b10)
                case (multiply_digit_0)
                4'd0:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_0 : 12'h0;
                4'd1:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_1 : 12'h0;
                4'd2:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_2 : 12'h0;
                4'd3:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_3 : 12'h0;
                4'd4:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_4 : 12'h0;
                4'd5:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_5 : 12'h0;
                4'd6:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_6 : 12'h0;
                4'd7:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_7 : 12'h0;
                4'd8:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_8 : 12'h0;
                4'd9:
                    {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel_9 : 12'h0;
                default: 
                    {vgaRed, vgaGreen, vgaBlue} = 12'h0;
                endcase
            else
                {vgaRed, vgaGreen, vgaBlue} = 12'h0;
        else
            {vgaRed, vgaGreen, vgaBlue} = 12'h0;
    end
endmodule



